/**
 * atk_feeds.spec.js
 *
 * Validate feeds entity.
 *
 */

/** ESLint directives */
/* eslint-disable import/first */

// Set up Playwright.
import { expect, test } from '@playwright/test'

import * as atkCommands from '../support/atk_commands'
import * as atkUtilities from '../support/atk_utilities'

import playwrightConfig from '../../playwright.config'

const baseUrl = playwrightConfig.use.baseURL // eslint-disable-line no-unused-vars

// Import ATK Configuration.
import atkConfig from '../../playwright.atk.config'

// Standard accounts that use user accounts created
// by QA Accounts. QA Accounts are created when the QA
// Accounts module is enabled.
import qaUserAccounts from '../data/qaUsers.json'

test.describe('Feeds.', () => {
  // Remember names for clean up.
  let feedTypeName

  //
  // Create a new feed definition, import fixture, validate that data is in the database.
  //
  test('(ATK-PW-1180) Feeds Test using Playwright. Tags : @ATK-PW-1180, @feeds, @alters-db', async ({ page, context }) => {
    await page.goto('/')
    await atkCommands.logInViaForm(page, context, qaUserAccounts.admin)
    //
    // Add a Feed type
    //
    await page.goto(atkConfig.feedTypeAddUrl)

    // Make a unique feed name.
    feedTypeName = `Import with url ${atkUtilities.createRandomString(10)}`

    // Fill in the fields
    await page.getByLabel('Name', { exact: true }).click()
    await page.getByLabel('Name', { exact: true }).fill(feedTypeName)
    await page.getByLabel('Description').click()
    await page.getByLabel('Description').fill('Import feeds')
    await page.getByLabel('Explanation or submission').click()
    await page.getByLabel('Explanation or submission').fill('Atk feeds test')

    // Save the configured feed type
    await page.getByRole('button', { name: 'Save and add mappings' }).click()
    await page.waitForEvent('domcontentloaded')

    //
    // Create mapping for imports
    //
    await page.getByLabel('Add a target').selectOption('title')
    await page.locator('[data-drupal-selector="edit-mappings-0-map-value-select"]').selectOption('title')
    await page.getByLabel('Add a target').selectOption('body')
    await page.locator('[data-drupal-selector="edit-mappings-1-map-value-select"]').selectOption('feed_description')
    await page.locator('[data-drupal-selector="edit-mappings-1-map-summary-select"]').selectOption('feed_title')
    await page.getByLabel('Add a target').selectOption('feeds_item')
    await page.locator('[data-drupal-selector="edit-mappings-2-map-url-select"]').selectOption('url')
    await page.locator('[data-drupal-selector="edit-mappings-2-map-guid-select"]').selectOption('guid')
    await page.getByRole('button', { name: 'Save' }).click()

    //
    // Add a new feed into the existing feed type.
    //
    await page.goto(atkConfig.feedAddUrl)

    // Find the existing feed type.
    // Select the feed that you have created.
    const regExpMatchArray = page.url().match(/feed\/add\/(\w+)/)
    if (regExpMatchArray) {
      // Creating feed for the only existing feed type.
    } else {
      await page.getByRole('link', { name: feedTypeName }).click()
    }
    await page.getByLabel('Title').click()
    await page.getByLabel('Title').fill('nytimes rss business')

    //
    // Insert path to Import file or the data file URL.
    //
    await page.getByLabel('Feed URL').click()
    await page.getByLabel('Feed URL').fill('https://rss.nytimes.com/services/xml/rss/nyt/Business.xml')

    // Save and export the feed.
    await page.getByRole('button', { name: 'Save and import' }).click()
    await page.getByText('nytimes rss business has been created.', { exact: true }).waitFor()

    // Validate that at least one article is created. (Author should be "Anonymous")
    await page.goto('admin/content')
    expect(await page.locator('tr', { hasText: 'Anonymous' }).count()).toBeGreaterThan(0)
  })

  test.afterEach(async ({ page }) => {
    //
    // Clean up the created entities.
    //

    // Delete feed items and delete a feed.
    await page.goto('admin/content/feed')
    const rowLocator = page.locator('tr', { hasText: feedTypeName })

    // Skip if the feed doesn't exist.
    if (await rowLocator.count()) {
      await rowLocator.locator('.dropbutton__toggle').click()
      await rowLocator.getByText('Delete items').click()
      await page.getByRole('button', { name: 'Delete items' }).click()
      await rowLocator.waitFor() // Should we navigate back to the feed row.

      await rowLocator.locator('.dropbutton__toggle').click()
      await rowLocator.getByText('Delete', { exact: true }).click()
      await page.getByRole('button', { name: 'Delete' }).click()
    }

    // Delete feed type.
    await page.goto('admin/structure/feeds')
    // Skip if the feed type doesn't exist.
    if (await rowLocator.count()) {
      await rowLocator.locator('.dropbutton__toggle').click()
      await rowLocator.getByText('Delete').click()
      await page.getByRole('button', { name: 'Delete' }).click()
    }
  })
})
